# -*- coding: UTF-8 -*-

from objects import *
from objects.extend_objects import *
from nes_common.sql_operation import _Delete
import nes_common.logger as logger
import sys
import traceback

log = None


def getTablesFromMethod(method):
    moduleTables = []
    if hasattr(method, '__module__') and method.__module__ in sys.modules and hasattr(sys.modules[method.__module__], 'tables'):
        moduleTables = sys.modules[method.__module__].tables
    return moduleTables


class CommandClr(object):
    def __init__(self, name, clr, mode='ANY'):
        def command(self, jsonCommand):
            if not self.checkMode(mode, command.__name__):
                return True, []
            item = None
            try:
                for item in clr():
                    self.executor.execute(item)
                    self.simpleCommands.add(name.replace('clear', 'add').replace('Inventory', 'InventItem'))
                return True, []
            except Exception as ex:
                #    self.executor.rollback()
                if item:
                    log.error('%s. %s' % (ex, item))
                    return False, [item.table]
                log.error('%s' % ex)
                return False, getTablesFromMethod(clr)
        command.__name__ = name
        setattr(Command, command.__name__, command)


class CommandAdd(object):
    def __init__(self, name, paramName, generate, delete, namedParameters=None, mode='ANY'):
        def command(self, jsonCommand):
            if not self.checkMode(mode, command.__name__):
                return True, []
            param = None
            try:
                param = jsonCommand[paramName]
            except:
                log.error("The name of the parameter '%s' not found." % paramName)
            kwargs = {}
            if namedParameters:
                for name in namedParameters:
                    val = jsonCommand.get(name, None)
                    if val:
                        kwargs[name] = val
            item = None
            try:
                # На кс все вставки теперь работают по insert into ... on duplicate, но команду вставки дисконта нужно все равно предварительно удялять,
                #т.к для нее особенное поведение - она одна
                if 'CS' not in self.modes or jsonCommand['command'] == 'replaceDiscountSystem':
                    for item in delete(param, **kwargs):
                        self.executor.execute(item)
                for item in generate(param, **kwargs):
                    # Так как для КС теперь используется insert into ... on duplicate, то для некоторых сущностей команда удаления должна выполняться
                    # непосредственно в Insert - команде. Для кассы используется схема "сперва удали, потом вставь новую", соответсвенно эту команду, хоть и
                    # можно выполнить, но она излишняя.
                    if isinstance(item, _Delete):
                        if 'CS' in self.modes:
                            self.executor.execute(item)
                            self.executor.commit()
                    else:
                        self.executor.execute(item)
                        self.executor.commit()
                return True, []
            except Exception as ex:
                if item:
                    log.error('%s. %s' % (traceback.format_exc(), item))
                    return False, [item.table]
                log.error('%s' % ex)
                #    self.executor.rollback()
                return False, getTablesFromMethod(generate)
        command.__name__ = name
        setattr(Command, command.__name__, command)


class CommandDel(object):
    def __init__(self, name, paramName, delete, namedParameters=None, mode='ANY'):
        def command(self, jsonCommand):
            if not self.checkMode(mode, command.__name__):
                return True, []
            param = jsonCommand[paramName]
            kwargs = {}
            if namedParameters:
                for name in namedParameters:
                    val = jsonCommand.get(name, None)
                    if val:
                        kwargs[name] = val
            item = None
            try:
                for item in delete(param, **kwargs):
                    self.executor.execute(item)
                self.executor.commit()
                return True, []
            except Exception as ex:
                #    self.executor.rollback()
                if item:
                    log.error('%s. %s' % (traceback.format_exc(), item))
                    return False, [item.table]
                log.error('%s' % (traceback.format_exc()))
                return False, getTablesFromMethod(delete)
        command.__name__ = name
        setattr(Command, command.__name__, command)


class CommandUpd(object):
    def __init__(self, name, paramName, update, namedParameters=None, mode='ANY'):
        def command(self, jsonCommand):
            if not self.checkMode(mode, command.__name__):
                return True, []
            param = jsonCommand[paramName]
            kwargs = {}
            if namedParameters:
                for name in namedParameters:
                    val = jsonCommand.get(name, None)
                    if val:
                        kwargs[name] = val
            item = None
            try:
                self.executor.executeMany()
                for item in update(param, **kwargs):
                    self.executor.execute(item)
                self.executor.commit()
                return True, []
            except Exception as ex:
                if item:
                    log.error('%s. %s' % (ex, item))
                    return False, [item.table]
                log.error('%s' % ex)
                return False, getTablesFromMethod(update)
        command.__name__ = name
        setattr(Command, command.__name__, command)


class Command(object):
    """
    NES command
    """

    def __init__(self, executor, shopCode, simpleCommands=set(), modes=['ANY']):
        global log
        log = logger.getLogger("Shop", shopCode)
        self.executor = executor
        self.simpleCommands = simpleCommands
        self.modes = modes

    def checkMode(self, mode, commandName):
        if mode not in self.modes:
            log.warning('Ignore comand %s for modes %s' % (commandName, self.modes))
            return False
        return True

    def dispatch(self, jsonCommand):
        """ returns method which can process jsonCommand or None if no such method """
        try:
            return getattr(self, jsonCommand['command'])
        except AttributeError:
            return None


CommandAdd(name='addAccountpincode', paramName='accountpincode', generate=accountpincode.insertAccountpincode, delete=accountpincode.deleteAccountpincodeReal, mode='CS_ALL')
CommandAdd(name='clearAccountpincode', paramName='serverid', generate=accountpincode.clearAccountpincode, delete=accountpincode.deleteAccountpincodePass, mode='CS_ALL')
CommandAdd(name='deleteAccountpincode', paramName='accountpincode', generate=accountpincode.deleteAccountpincode, delete=accountpincode.deleteAccountpincodeReal, mode='CS_ALL')

CommandAdd(name='addTmc', paramName='invent', generate=tmc.insertTmc, delete=tmc.deleteTmc)
CommandClr(name='clearTmc', clr=tmc.clearTmc)
CommandDel(name='deleteTmc', paramName='inventcode', delete=tmc.deleteTmcByCode)

CommandAdd(name='addInventItem', paramName='invent',
             generate=invent.insertInventItem, delete=invent.deleteInventItem)
CommandClr(name='clearInventory', clr=invent.clearInventory)
CommandDel(name='deleteInventItem', paramName='inventcode', delete=invent.deleteInventItemByCode)
CommandUpd(name='updateInventItem', paramName='invent', update=invent.updateInventItem)

CommandAdd(name='addLDiscCard', paramName='ldisccard',
             generate=ldisccard.insertLDiscCard, delete=ldisccard.deleteLDiscCard)
CommandClr(name='clearLDiscCard', clr=ldisccard.clearLDiscCard)
CommandDel(name='deleteLDiscCard', paramName='cardnumber', delete=ldisccard.deleteLDiscCardByNumber)

CommandAdd(name='addLDiscFixed', paramName='ldiscfixed',
             generate=ldiscfixed.insertLDiscFixed, delete=ldiscfixed.deleteLDiscFixed)
CommandClr(name='clearLDiscFixed', clr=ldiscfixed.clearLDiscFixed)
CommandDel(name='deleteLDiscFixed', paramName='disccode', delete=ldiscfixed.deleteLDiscFixedByCode)

CommandAdd(name='addLDiscAccumulate', paramName='ldiscaccumulate',
             generate=ldiscaccumulate.insertLDiscAccumulate, delete=ldiscaccumulate.deleteLDiscAccumulate)
CommandClr(name='clearLDiscAccumulate', clr=ldiscaccumulate.clearLDiscAccumulate)
CommandDel(name='deleteLDiscAccumulate', paramName='discscheme', delete=ldiscaccumulate.deleteLDiscAccumulateByCode)

CommandAdd(name='addDiscCardAccumulate', paramName='disccardaccumulate',
             generate=disccardaccumulate.insertDiscCardAccumulate, delete=disccardaccumulate.deleteDiscCardAccumulate)
CommandClr(name='clearDiscCardAccumulate', clr=disccardaccumulate.clearDiscCardAccumulate)
CommandDel(name='deleteDiscCardAccumulate', paramName='disccode', delete=disccardaccumulate.deleteDiscCardAccumulateByDiscCode)

CommandAdd(name='addAccumulateSPrices', paramName='accumulatesprices',
             generate=accumulatesprices.insertAccumulateSPrices, delete=accumulatesprices.deleteAccumulateSPrices)
CommandClr(name='clearAccumulateSPrices', clr=accumulatesprices.clearAccumulateSPrices)
CommandDel(name='deleteAccumulateSPrices', paramName='code', delete=accumulatesprices.deleteAccumulateSPricesByCode)

CommandAdd(name='addLDiscScheme', paramName='ldiscscheme',
            generate=ldiscscheme.insertLDiscScheme, delete=ldiscscheme.deleteLDiscScheme)
CommandClr(name='clearLDiscScheme', clr=ldiscscheme.clearLDiscScheme)
CommandDel(name='deleteLDiscScheme', paramName='schemecode', delete=ldiscscheme.deleteLDiscSchemeByCode)

CommandAdd(name='addDiscountAction', paramName='discountaction',
            generate=discountaction.insertDiscountAction, delete=discountaction.deleteDiscountAction)
CommandClr(name='clearDiscountAction', clr=discountaction.clearDiscountAction)
CommandDel(name='deleteDiscountAction', paramName='campaignid', delete=discountaction.deleteDiscountActionById)

CommandAdd(name='addTaxGroup', paramName='taxgroup',
            generate=tax.insertTaxGroup, delete=tax.deleteTaxGroup)
CommandClr(name='clearTaxGroup', clr=tax.clearTaxGroup)
CommandDel(name='deleteTaxGroup', paramName='idtaxgroup', delete=tax.deleteTaxGroupByCode)
CommandUpd(name='updateTaxGroup', paramName='tax', update=tax.updateTaxGroup)

CommandAdd(name='addCashUser', paramName='cashuser',
            generate=cashuser.insertCashUser, delete=cashuser.deleteCashUser)
CommandClr(name='clearCashUser', clr=cashuser.clearCashUser)
CommandDel(name='deleteCashUser', paramName='usercode', delete=cashuser.deleteCashUserByCode)

CommandAdd(name='replaceDiscountSystem', paramName='discsystemdesc',
             generate=discountsystem.replaceDiscountSystem, delete=discountsystem.deleteDiscountSystem,
             namedParameters=['limitbonussystemdesc', 'earnbonussystemdesc', 'seconddiscsystemdesc'])

CommandAdd(name='addUnit', paramName='unit', generate=units.insertUnit, delete=units.deleteUnit)
CommandClr(name='clearUnit', clr=units.clearUnit)
CommandDel(name='deleteUnit', paramName='unitcode', delete=units.deleteUnitByCode)

CommandAdd(name='addMeasure', paramName='measure',
            generate=measure.insertMeasure, delete=measure.deleteMeasure)
CommandClr(name='clearMeasure', clr=measure.clearMeasure)
CommandDel(name='deleteMeasure', paramName='idmeasure', delete=measure.deleteMeasureByCode)

CommandAdd(name='addCard', paramName='card', generate=card.insertCard, delete=card.deleteCard)
CommandClr(name='clearCard', clr=card.clearCard)
CommandDel(name='deleteCard', paramName='idcard', delete=card.deleteCardById)
CommandUpd(name='updateCard', paramName='card', update=card.updateCard)

CommandAdd(name='addCardGroup', paramName='cardgroup', generate=cardgroup.insertCardGroup, delete=cardgroup.deleteCardGroup)
CommandClr(name='clearCardGroup', clr=cardgroup.clearCardGroup)
CommandDel(name='deleteCardGroup', paramName='idcardgroup', delete=cardgroup.deleteCardGroupById)

CommandAdd(name='addClient', paramName='client', generate=client.insertClient, delete=client.deleteClient)
CommandClr(name='clearClient', clr=client.clearClient)
CommandDel(name='deleteClient', paramName='idclient', delete=client.deleteClientById)
CommandUpd(name='updateClient', paramName='client', update=client.updateClient)

CommandAdd(name='addOrganization', paramName='organization', generate=organization.insertOrganization, delete=organization.deleteOrganization)
CommandClr(name='clearOrganization', clr=organization.clearOrganization)
CommandDel(name='deleteOrganization', paramName='organizationcode', delete=organization.deleteOrganizationByCode)

CommandAdd(name='addLVat', paramName='lvat', generate=lvat.insertLVat, delete=lvat.deleteLVat)
CommandClr(name='clearLVat', clr=lvat.clearLVat)
CommandDel(name='deleteLVat', paramName='code', delete=lvat.deleteLVatByCode)
CommandUpd(name='updateLVat', paramName='lvat', update=lvat.updateLVat)

CommandAdd(name='addInventGroup', paramName='inventgroup', generate=inventgroup.insertInventGroup, delete=inventgroup.deleteInventGroup)
CommandClr(name='clearInventGroup', clr=inventgroup.clearInventGroup)
CommandDel(name='deleteInventGroup', paramName='groupcode', delete=inventgroup.deleteInventGroupByCode)

CommandAdd(name='addInventCatalog', paramName='inventcatalog', generate=inventcatalog.insertInventCatalog, delete=inventcatalog.deleteInventCatalog)
CommandClr(name='clearInventCatalog', clr=inventcatalog.clearInventCatalog)
CommandDel(name='deleteInventCatalog', paramName='inventcatalog', delete=inventcatalog.deleteInventCatalog)

CommandAdd(name='addCatalog', paramName='catalog', generate=catalog.insertCatalog, delete=catalog.deleteCatalog)
CommandClr(name='clearCatalog', clr=catalog.clearCatalog)
CommandDel(name='deleteCatalog', paramName='catalogid', delete=catalog.deleteCatalogById)
CommandDel(name='deleteCatalogAndTmcCatalog', paramName='catalogid', delete=catalog.deleteCatalogAndTmcCatalog)

CommandAdd(name='addCatalogAttribute', paramName='catalogattribute', generate=catalogattribute.insertCatalogAttribute, delete=catalogattribute.deleteCatalogAttribute)
CommandClr(name='clearCatalogAttribute', clr=catalogattribute.clearCatalogAttribute)
CommandDel(name='deleteCatalogAttribute', paramName='catalogid', delete=catalogattribute.deleteCatalogAttributeById)

CommandAdd(name='addChangeAccountBalance', paramName='changeaccountbalance', generate=changeaccountbalance.insertChangeAccountBalance, delete=changeaccountbalance.deleteChangeAccountBalance, mode='CS_ALL')
CommandClr(name='clearChangeAccountBalance', clr=changeaccountbalance.clearChangeAccountBalance, mode='CS_ALL')
CommandDel(name='deleteChangeAccountBalance', paramName='sessionid', delete=changeaccountbalance.deleteChangeAccountBalanceById, mode='CS_ALL')

CommandAdd(name='addChangeCardAccount', paramName='changecardaccount', generate=changecardaccount.insertChangeCardAccount, delete=changecardaccount.deleteChangeCardAccount, mode='CS_ALL')
CommandClr(name='clearChangeCardAccount', clr=changecardaccount.clearChangeCardAccount, mode='CS_ALL')
CommandDel(name='deleteChangeCardAccount', paramName='cardnumber', delete=changecardaccount.deleteChangeCardAccountById, mode='CS_ALL')

CommandAdd(name='addCurrency', paramName='currency', generate=valut.insertValut, delete=valut.deleteValut)
CommandClr(name='clearCurrency', clr=valut.clearValut)
CommandDel(name='deleteCurrency', paramName='currencycode', delete=valut.deleteValutByCode)

CommandAdd(name='addCommand', paramName='commandobject', generate=command.insertCommand, delete=command.deleteCommand)
CommandClr(name='clearCommand', clr=command.clearCommand)
CommandDel(name='deleteCommand', paramName='commandcode', delete=command.deleteCommandByCode)

CommandAdd(name='addKbKey', paramName='kbkey', generate=kbkey.insertKbKey, delete=kbkey.deleteKbKey)
CommandClr(name='clearKbKey', clr=kbkey.clearKbKey)
CommandDel(name='deleteKbKey', paramName='keycode', delete=kbkey.deleteKbKeyByCode, namedParameters=['context'])

CommandAdd(name='addButton', paramName='button', generate=button.insertButton, delete=button.deleteButton)
CommandClr(name='clearButton', clr=button.clearButton)
CommandDel(name='deleteButton', paramName='code', delete=button.deleteButtonByCode,)

CommandAdd(name='addDept', paramName='dept', generate=dept.insertDept, delete=dept.deleteDept)
CommandClr(name='clearDept', clr=dept.clearDept)
CommandDel(name='deleteDept', paramName='code', delete=dept.deleteDeptByCode,)

CommandAdd(name='addMCashUser', paramName='mcashuser', generate=mcashuser.insertMCashUser, delete=mcashuser.deleteMCashUser)
CommandClr(name='clearMCashUser', clr=mcashuser.clearMCashUser)
CommandDel(name='deleteMCashUser', paramName='usercode', delete=mcashuser.deleteMCashUserByCode)

CommandAdd(name='addCmaction', paramName='cmaction', generate=cmaction.insertCmaction, delete=cmaction.deleteCmaction)
CommandClr(name='clearCmaction', clr=cmaction.clearCmaction)
CommandDel(name='deleteCmaction', paramName='cmactioncode', delete=cmaction.deleteCmactionByCode)

CommandAdd(name='addMenu', paramName='menu', generate=menu.insertMenu, delete=menu.deleteMenu)
CommandClr(name='clearMenu', clr=menu.clearMenu)
CommandDel(name='deleteMenu', paramName='menucode', delete=menu.deleteMenuByCode)

CommandAdd(name='addRole', paramName='role', generate=role.insertRole, delete=role.deleteRole)
CommandClr(name='clearRole', clr=role.clearRole)
CommandDel(name='deleteRole', paramName='rolecode', delete=role.deleteRoleByCode)

CommandAdd(name='addAspect', paramName='aspect', generate=aspect.insertAspect, delete=aspect.deleteAspect)
CommandClr(name='clearAspect', clr=aspect.clearAspect)
CommandDel(name='deleteAspect', paramName='aspectcode', delete=aspect.deleteAspectByCode)

CommandAdd(name='addAspectScheme', paramName='aspectscheme', generate=aspectscheme.insertAspectScheme, delete=aspectscheme.deleteAspectScheme)
CommandClr(name='clearAspectScheme', clr=aspectscheme.clearAspectScheme)
CommandDel(name='deleteAspectScheme', paramName='aspectschemecode', delete=aspectscheme.deleteAspectSchemeByCode)

CommandAdd(name='addAspectValue', paramName='aspectvalue', generate=aspectvalue.insertAspectValue, delete=aspectvalue.deleteAspectValue)
CommandClr(name='clearAspectValue', clr=aspectvalue.clearAspectValue)
CommandDel(name='deleteAspectValue', paramName='aspectvaluecode', delete=aspectvalue.deleteAspectValueByCode)

CommandAdd(name='addAspectValueSet', paramName='aspectvalueset', generate=aspectvalueset.insertAspectValueSet, delete=aspectvalueset.deleteAspectValueSet)
CommandClr(name='clearAspectValueSet', clr=aspectvalueset.clearAspectValueSet)
CommandDel(name='deleteAspectValueSet', paramName='aspectvaluesetcode', delete=aspectvalueset.deleteAspectValueSetByCode)
CommandUpd(name='updateAspectValueSet', paramName='aspectvalueset', update=aspectvalueset.increaseAspectValueSetRemain)

CommandAdd(name='addAspectValueSetValues', paramName='aspectvaluesetvalues', generate=aspectvaluesetvalues.insertAspectValueSetValues, delete=aspectvaluesetvalues.deleteAspectValueSetValues)
CommandClr(name='clearAspectValueSetValues', clr=aspectvaluesetvalues.clearAspectValueSetValues)
CommandDel(name='deleteAspectValueSetValues', paramName='aspectvaluesetvalues', delete=aspectvaluesetvalues.deleteAspectValueSetValues)

CommandAdd(name='addHotKey', paramName='hotkey', generate=hotkey.insertHotKey, delete=hotkey.deleteHotKey)
CommandClr(name='clearHotKey', clr=hotkey.clearHotKey)
CommandDel(name='deleteHotKey', paramName='hotkeycode', delete=hotkey.deleteHotKeyByCode)

CommandAdd(name='addBarcode', paramName='barcode', generate=barcode.insertBarcode, delete=barcode.deleteBarcode)
CommandClr(name='clearBarcode', clr=barcode.clearBarcode)
CommandDel(name='deleteBarcode', paramName='barcode', delete=barcode.deleteBarcodeByBarcode)

CommandAdd(name='addPrice', paramName='price', generate=price.insertPrice, delete=price.deletePrice)
CommandClr(name='clearPrice', clr=price.clearPrice)
CommandDel(name='deletePrice', paramName='barcode', delete=price.deletePriceByBarcode, namedParameters=['documentid', 'doctype'])
CommandDel(name='deletePriceByDate', paramName='barcode', delete=price.deletePriceByDate, namedParameters=['effectivedate', 'effectivedateend'])
CommandDel(name='deleteOverduePrices', paramName='barcode', delete=price.deleteOverduePrices, namedParameters=[])

CommandAdd(name='addRemain', paramName='remain', generate=remain.insertRemain, delete=remain.deleteRemainByObj, mode='CS')
CommandClr(name='clearRemain', clr=remain.clearRemain, mode='CS')
CommandDel(name='deleteRemain', paramName='documentid', delete=remain.deleteRemain, namedParameters=['barcode', 'changetype'], mode='CS')
CommandDel(name='deleteOutdatedRemains', paramName='barcode', delete=remain.deleteRemainsBeforeDate, namedParameters=['effectivedate'], mode='CS')

CommandAdd(name='addShop', paramName='shop', generate=shop.insertShop, delete=shop.deleteShop, mode='CASH')
CommandClr(name='clearShop', clr=shop.clearShop, mode='CASH')
CommandDel(name='deleteShop', paramName='code', delete=shop.deleteShopByCode, mode='CASH')

CommandAdd(name='addAgerestrict', paramName='agerestrict', generate=agerestrict.insertAgeRestrict, delete=agerestrict.deleteAgeRestrict)
CommandClr(name='clearAgerestrict', clr=agerestrict.clearAgeRestrict)
CommandDel(name='deleteAgerestrict', paramName='alcoholpercent', delete=agerestrict.deleteAgeRestrictByAlcoholPercent)

CommandAdd(name='addTimerestrict', paramName='timerestrict', generate=timerestrict.insertTimeRestrict, delete=timerestrict.deleteTimeRestrict)
CommandClr(name='clearTimerestrict', clr=timerestrict.clearTimeRestrict)
CommandDel(name='deleteTimerestrict', paramName='timerestrictid', delete=timerestrict.deleteTimeRestrictByTimeRestrictId)

CommandAdd(name='addTmcrequiredquantity', paramName='tmcrequiredquantity', generate=inventrequiredquantity.insertInventRequiredQuantity, delete=inventrequiredquantity.deleteInventRequiredQuantity)
CommandClr(name='clearTmcrequiredquantity', clr=inventrequiredquantity.clearInventRequiredQuantity)
CommandDel(name='deleteTmcrequiredquantity', paramName='tmccode', delete=inventrequiredquantity.deleteInventRequiredQuantity)

CommandAdd(name='addTmcvisualverify', paramName='tmcvisualverify', generate=inventvisualverify.insertInventVisualVerify, delete=inventvisualverify.deleteInventVisualVerify)
CommandClr(name='clearTmcvisualverify', clr=inventvisualverify.clearInventVisualVerify)
CommandDel(name='deleteTmcvisualverify', paramName='tmccode', delete=inventvisualverify.deleteInventVisualVerify)

CommandAdd(name='addTmcScale', paramName='tmcscale', generate=tmcscale.insertTmcScale, delete=tmcscale.deleteTmcScaleByObj, mode='CS')
CommandClr(name='clearTmcScale', clr=tmcscale.clearTmcScale, mode='CS')
CommandDel(name='deleteTmcScale', paramName='tmccode', delete=tmcscale.deleteTmcScale, namedParameters=['tmcscalecode'], mode='CS')

CommandAdd(name='addTmcScaleGroup', paramName='tmcscalegroup', generate=tmcscalegroup.insertTmcScaleGroup, delete=tmcscalegroup.deleteTmcScaleGroupByObj, mode='CS')
CommandClr(name='clearTmcScaleGroup', clr=tmcscalegroup.clearTmcScaleGroup, mode='CS')
CommandDel(name='deleteTmcScaleGroup', paramName='tmcscalegroupcode', delete=tmcscalegroup.deleteTmcScaleGroup, mode='CS')

CommandAdd(name='addMacro', paramName='macro', generate=macro.insertMacro, delete=macro.deleteMacro)
CommandClr(name='clearMacro', clr=macro.clearMacro)
CommandDel(name='deleteMacro', paramName='macrocode', delete=macro.deleteMacroByCode)

CommandAdd(name='addShopLabels', paramName='group', generate=shoplabels.insertShopLabels, delete=shoplabels.deleteShopLabels, mode='CASH')
CommandClr(name='clearShopLabels', clr=shoplabels.clearShopLabels, mode='CASH')
CommandDel(name='deleteShopLabels', paramName='groupcode', delete=shoplabels.deleteShopLabelsByCode, mode='CASH')

CommandAdd(name='addSoftCheck', paramName='softcheck', generate=softcheck.insertSoftCheck, delete=softcheck.deleteSoftCheck, mode='CS_ALL')
CommandClr(name='clearSoftCheck', clr=softcheck.clearSoftCheck, mode='CS_ALL')
CommandDel(name='deleteSoftCheck', paramName='softcheckid', delete=softcheck.deleteSoftCheckById, mode='CS_ALL')

# extend_objects
CommandAdd(name='addHotKeyInvent', paramName='hotkeyinvent', generate=hotkeyinvent.insertHotKeyInvent, delete=hotkeyinvent.deleteHotKeyInvent)
CommandClr(name='clearHotKeyInvent', clr=hotkeyinvent.clearHotKeyInvent)
CommandDel(name='deleteHotKeyInvent', paramName='hotkeyinvent', delete=hotkeyinvent.deleteHotKeyInvent)

CommandAdd(name='addDiscountsAuto', paramName='discountsauto', generate=discountsauto.insertDiscountsauto, delete=discountsauto.deleteDiscountsauto)
CommandClr(name='clearDiscountsAuto', clr=discountsauto.clearDiscountsauto)
CommandDel(name='deleteDiscountsAuto', paramName='discountsauto', delete=discountsauto.deleteDiscountsauto)

CommandAdd(name='addTimePeriod', paramName='timeperiod', generate=timeperiod.insertTimePeriod, delete=timeperiod.deleteTimePeriod)
CommandClr(name='clearTimePeriod', clr=timeperiod.clearTimePeriod)
CommandDel(name='deleteTimePeriod', paramName='timeperiod', delete=timeperiod.deleteTimePeriod)

CommandAdd(name='addKit', paramName='kit', generate=kit.insertKit, delete=kit.deleteKit)
CommandClr(name='clearKit', clr=kit.clearKit)
CommandDel(name='deleteKit', paramName='kit', delete=kit.deleteKit)

CommandAdd(name='addKitInvent', paramName='kitinvent', generate=kitinvent.insertKitInvent, delete=kitinvent.deleteKitInvent)
CommandClr(name='clearKitInvent', clr=kitinvent.clearKitInvent)
CommandDel(name='deleteKitInvent', paramName='kitinvent', delete=kitinvent.deleteKitInvent)

CommandAdd(name='addKitDisc', paramName='kitdisc', generate=kitdisc.insertKitDisc, delete=kitdisc.deleteKitDisc)
CommandClr(name='clearKitDisc', clr=kitdisc.clearKitDisc)
CommandDel(name='deleteKitDisc', paramName='kitdisc', delete=kitdisc.deleteKitDisc)

CommandAdd(name='addAdditionalPrice', paramName='additionalprice', generate=additionalprice.insertTprices, delete=additionalprice.deleteTprices)
CommandClr(name='clearAdditionalPrice', clr=additionalprice.clearTprices)
CommandDel(name='deleteAdditionalPrice', paramName='additionalprice', delete=additionalprice.deleteTprices)
CommandDel(name='deleteOutdatedAdditionalPrice', paramName='barcode', delete=additionalprice.deleteOutdatedTprices,
           namedParameters=['pricecode', 'effectivedate'])
CommandDel(name='deleteFutureAdditionalPrice', paramName='barcode', delete=additionalprice.deleteFutureTprices,
           namedParameters=['pricecode', 'effectivedate'])

CommandAdd(name='addCertificate', paramName='certificate', generate=certificate.insertCertificate, delete=certificate.deleteCertificate)
CommandClr(name='clearCertificate', clr=certificate.clearCertificate)
CommandDel(name='deleteCertificate', paramName='code', delete=certificate.deleteCertificateByCode)

CommandAdd(name='addSaleRestrict', paramName='salerestrict', generate=salerestrict.insertSaleRestrict, delete=salerestrict.deleteSaleRestrict)
CommandClr(name='clearSaleRestrict', clr=salerestrict.clearSaleRestrict)
CommandDel(name='deleteSaleRestrict', paramName='salerestrictcode', delete=salerestrict.deleteBySaleRestrictCode)

CommandAdd(name='addConsultant', paramName='consultant', generate=consultant.insertConsultant, delete=consultant.deleteConsultant)
CommandClr(name='clearConsultant', clr=consultant.clearConsultant)
CommandDel(name='deleteConsultant', paramName='code', delete=consultant.deleteConsultantByCode)

CommandAdd(name='addBackReason', paramName='backreason', generate=backreason.insertBackReason, delete=backreason.deleteBackReason)
CommandClr(name='clearBackReason', clr=backreason.clearBackReason)
CommandDel(name='deleteBackReason', paramName='code', delete=backreason.deleteBackReasonByCode)

CommandAdd(name='addCorrectionReason', paramName='correctionreason', generate=correctionreason.insertCorrectionReason, delete=correctionreason.deleteCorrectionReason)
CommandClr(name='clearCorrectionReason', clr=correctionreason.clearCorrectionReason)
CommandDel(name='deleteCorrectionReason', paramName='code', delete=correctionreason.deleteCorrectionReasonByCode)

CommandAdd(name='addAlcoCode', paramName='alcocode', generate=alcocode.insertAlcoCode, delete=alcocode.deleteAlcoCode)
CommandClr(name='clearAlcoCode', clr=alcocode.clearAlcoCode)
CommandDel(name='deleteAlcoCode', paramName='alcocode', delete=alcocode.deleteAlcoCodeByCode)

CommandAdd(name='addTmcAlcocode', paramName='tmcalcocode', generate=tmcalcocode.insertTmcAlcocode, delete=tmcalcocode.deleteTmcAlcocode)
CommandClr(name='clearTmcAlcocode', clr=tmcalcocode.clearTmcAlcocode)
CommandDel(name='deleteTmcAlcocode', paramName='tmcalcocode', delete=tmcalcocode.deleteTmcAlcocode)

CommandAdd(name='addEvent', paramName='event', generate=event.insertEvent, delete=event.deleteEvent)
CommandClr(name='clearEvent', clr=event.clearEvent)
CommandDel(name='deleteEvent', paramName='eventname', delete=event.deleteEventByName)

CommandAdd(name='addExciseMark', paramName='excisemark', generate=excisemark.insertExciseMark, delete=excisemark.deleteExciseMark, mode='CS_ALL')
CommandClr(name='clearExciseMark', clr=excisemark.clearExciseMark, mode='CS_ALL')
CommandDel(name='deleteExciseMark', paramName='excisemarkid', namedParameters=['excisetype'], delete=excisemark.deleteExciseMarkById, mode='CS_ALL')

CommandAdd(name='addExciseMarkWhite', paramName='excisemarkwhite', generate=excisemarkwhite.insertExciseMarkWhite, delete=excisemarkwhite.deleteExciseMarkWhite)
CommandAdd(name='addExciseMarksWhite', paramName='excisemarkswhite', generate=excisemarkwhite.insertExciseMarksWhite, delete=excisemarkwhite.deleteExciseMarksWhite)
CommandClr(name='clearExciseMarkWhite', clr=excisemarkwhite.clearExciseMarkWhite)
CommandDel(name='deleteExciseMarkWhite', paramName='excisemarkid', namedParameters=['barcode', 'excisetype'], delete=excisemarkwhite.deleteExciseMarkWhiteById)

CommandAdd(name='addAlcoSet', paramName='alcoset',
            generate=alcoset.insertAlcoSet, delete=alcoset.deleteAlcoSet)
CommandClr(name='clearAlcoSet', clr=alcoset.clearAlcoSet)
CommandDel(name='deleteAlcoSet', paramName='alcoset', delete=alcoset.deleteAlcoSet)

CommandAdd(name='addAdditionalCard', paramName='additionalcard',
            generate=additionalcard.insertAdditionalCard, delete=additionalcard.deleteAdditionalCard, mode='CS_ALL')
CommandClr(name='clearAdditionalCard', clr=additionalcard.clearAdditionalCard, mode='CS_ALL')
CommandDel(name='deleteAdditionalCard', paramName='additionalcard', delete=additionalcard.deleteAdditionalCard, mode='CS_ALL')

CommandAdd(name='addCardGroupSearchPlace', paramName='cardgroupsearchplace',
            generate=cardgroupsearchplace.insertCardGroupSearchPlace, delete=cardgroupsearchplace.deleteCardGroupSearchPlace, mode='CS_ALL')
CommandClr(name='clearCardGroupSearchPlace', clr=cardgroupsearchplace.clearCardGroupSearchPlace, mode='CS_ALL')
CommandDel(name='deleteCardGroupSearchPlace', paramName='idcardgroup', delete=cardgroupsearchplace.deleteCardGroupSearchPlaceById, mode='CS_ALL')

CommandAdd(name='addVerification', paramName='verification', generate=verification.insertVerification, delete=verification.deleteVerification)
CommandClr(name='clearVerification', clr=verification.clearVerification)
CommandDel(name='deleteVerification', paramName='idverification', delete=verification.deleteVerificationById)

CommandAdd(name='addCardGroupVerification', paramName='cardgroupverification', generate=cardgroupverification.insertCardGroupVerification,
        delete=cardgroupverification.deleteCardGroupVerification)
CommandClr(name='clearCardGroupVerification', clr=cardgroupverification.clearCardGroupVerification)
CommandDel(name='deleteCardGroupVerification', paramName='cardgroupverification', delete=cardgroupverification.deleteCardGroupVerification)

CommandAdd(name='addCashConfig', paramName='cashconfig', generate=cashconfig.insertCashConfig, delete=cashconfig.deleteCashConfig)
CommandClr(name='clearCashConfig', clr=cashconfig.clearCashConfig)
CommandDel(name='deleteCashConfig', paramName='name', delete=cashconfig.deleteCashConfigByName)

CommandAdd(name='addActionPanel', paramName='actionpanel', generate=actionpanel.insertActionPanel, delete=actionpanel.deleteActionPanel)
CommandClr(name='clearActionPanel', clr=actionpanel.clearActionPanel)
CommandDel(name='deleteActionPanel', paramName='actionpanelcode', delete=actionpanel.deleteActionPanelByCode)

CommandAdd(name='addCountry', paramName='country', generate=country.insertCountry, delete=country.deleteCountry)
CommandClr(name='clearCountry', clr=country.clearCountry)
CommandDel(name='deleteCountry', paramName='code', delete=country.deleteCountryByCode)

CommandAdd(name='addMedicine', paramName='medicine', generate=medicine.insertMedicine, delete=medicine.deleteMedicine)
CommandClr(name='clearMedicine', clr=medicine.clearMedicine)
CommandDel(name='deleteMedicine', paramName='code', delete=medicine.deleteMedicineByCode)

CommandAdd(name='addSupplier', paramName='supplier', generate=supplier.insertSupplier, delete=supplier.deleteSupplier)
CommandClr(name='clearSupplier', clr=supplier.clearSupplier)
CommandDel(name='deleteSupplier', paramName='code', delete=supplier.deleteSupplierByCode)

CommandAdd(name='addTmcSupplier', paramName='tmcsupplier', generate=tmcsupplier.insertTmcSupplier, delete=tmcsupplier.deleteTmcSupplier)
CommandClr(name='clearTmcSupplier', clr=tmcsupplier.clearTmcSupplier)
CommandDel(name='deleteTmcSupplier', paramName='tmcsupplier', delete=tmcsupplier.deleteTmcSupplier)
CommandDel(name='deleteTmcSuppliersBySomeField', paramName='tmcsupplier', delete=tmcsupplier.deleteTmcSuppliersBySomeField)

CommandAdd(name='addMCashUserAccess', paramName='mcashuseraccess', generate=mcashuseraccess.insertMCashUserAccess, delete=mcashuseraccess.deleteMCashUserAccess)
CommandClr(name='clearMCashUserAccess', clr=mcashuseraccess.clearMCashUserAccess)
CommandDel(name='deleteMCashUserAccess', paramName='mcashuseraccess', delete=mcashuseraccess.deleteMCashUserAccess)

CommandAdd(name='addPicklist', paramName='picklist', generate=picklist.insertPicklist, delete=picklist.deletePicklist)
CommandClr(name='clearPicklist', clr=picklist.clearPicklist)
CommandDel(name='deletePicklist', paramName='code', delete=picklist.deletePicklistByCode)

CommandAdd(name='addRemainTmc', paramName='remaintmc', generate=remaintmc.insertRemainTmc, delete=remaintmc.deleteRemainTmc, mode='CS_ALL')
CommandClr(name='clearRemainTmc', clr=remaintmc.clearRemainTmc, mode='CS_ALL')
CommandDel(name='deleteRemainTmc', paramName='remaintmc', delete=remaintmc.deleteRemainTmc, mode='CS_ALL')

CommandAdd(name='addRemainBarcode', paramName='remainbarcode', generate=remainbarcode.insertRemainBarcode, delete=remainbarcode.deleteRemainBarcode)
CommandClr(name='clearRemainBarcode', clr=remainbarcode.clearRemainBarcode)
CommandDel(name='deleteRemainBarcode', paramName='barcode', delete=remainbarcode.deleteRemainBarcodeByBarcode)

CommandAdd(name='addRemainDraftBeer', paramName='remaindraftbeer', generate=remaindraftbeer.insertRemainDraftBeer, delete=remaindraftbeer.deleteRemainDraftBeer)
CommandClr(name='clearRemainDraftBeer', clr=remaindraftbeer.clearRemainDraftBeer)
CommandDel(name='deleteRemainDraftBeer', paramName='markingcode', delete=remaindraftbeer.deleteRemainDraftBeerByMarkingCode)

CommandAdd(name='addPersonalOffer', paramName='personaloffer', generate=personaloffer.insertPersonalOffer, delete=personaloffer.deletePersonalOffer, mode='CS_ALL')
CommandClr(name='clearPersonalOffer', clr=personaloffer.clearPersonalOffer, mode='CS_ALL')
CommandDel(name='deletePersonalOffer', paramName='personaloffer', delete=personaloffer.deletePersonalOffer, mode='CS_ALL')

CommandAdd(name='addCatalogRestrict', paramName='catalogrestrict', generate=catalogrestrict.insertCatalogRestrict, delete=catalogrestrict.deleteCatalogRestrict)
CommandClr(name='clearCatalogRestrict', clr=catalogrestrict.clearCatalogRestrict)
CommandDel(name='deleteCatalogRestrict', paramName='catalogrestrict', delete=catalogrestrict.deleteCatalogRestrict)

CommandAdd(name='addAgent', paramName='agent', generate=agent.insertAgent, delete=agent.deleteAgent)
CommandClr(name='clearAgent', clr=agent.clearAgent)
CommandDel(name='deleteAgent', paramName='code', delete=agent.deleteAgentByCode)

CommandAdd(name='addCategory', paramName='category', generate=category.insertCategory, delete=category.deleteCategory)
CommandClr(name='clearCategory', clr=category.clearCategory)
CommandDel(name='deleteCategory', paramName='categoryid', delete=category.deleteCategoryById)

CommandAdd(name='addTmcKit', paramName='tmckit', generate=tmckit.insertTmcKit, delete=tmckit.deleteTmcKit)
CommandClr(name='clearTmcKit', clr=tmckit.clearTmcKit)
CommandDel(name='deleteTmcKit', paramName='barcode', delete=tmckit.deleteTmcKitByBarcode)

CommandAdd(name='addTmcKitItem', paramName='tmckititem', generate=tmckititem.insertTmcKitItem, delete=tmckititem.deleteTmcKitItem)
CommandClr(name='clearTmcKitItem', clr=tmckititem.clearTmcKitItem)
CommandDel(name='deleteTmcKitItem', paramName='tmckititem', delete=tmckititem.deleteTmcKitItem)
