# -*- coding: UTF-8 -*-

from nes_common.config import config
import base64
import httplib2
import json
import nes_common.logger as logger
import traceback

STATUS_ERROR = 'STATUS_ERROR'                       # Ошибка загрузки/выгрузки справочника
STATUS_READY = 'STATUS_READY'                       # Справочник успешно загружен/выгружен
STATUS_READY_WITH_ERROR = 'STATUS_READY_WITH_ERROR' # Справочник загружен с ошибками
STATUS_DURING = 'STATUS_DURING'                     # Справочник в данный момент загружается/выгружается

cs_prefix = config.getString('CS', 'serverUrlPrefix', '/CSrest/rest')
cs_login = config.getString('CS', 'login', 'admin')
cs_password = config.getString('CS', 'passwd', 'admin')

requestTimeout = 60
auth = base64.b64encode('{0}:{1}'.format(cs_login, cs_password).encode('utf-8')).decode()
jsonHeader = {'Content-Type': 'application/json', 'Accept': 'application/json', 'Authorization': 'Basic %s' % auth}


class LoadDictStatusSender(object):
    def __init__(self, shopCode, dictCSLoadStatusId=None, dictCashLoadStatusId=None, statusServer='localhost:8080',
                 cashCode=None, version=None, isSendLoadDictStatus=True, isSendDebugDictStatus=True, defaultStatusServer='localhost:8080'):
        self.shopCode = shopCode
        self.dictCSLoadStatusId = dictCSLoadStatusId
        self.dictCashLoadStatusId = dictCashLoadStatusId
        # адрес сервера, для случая когда кассе надо проинформировать о своем прогрессе         
        self.statusServer = statusServer
        self.cashCode = cashCode
        self.log = logger.getLogger('Shop', shopCode)
        self.version = version
        self.isSendLoadDictStatus = isSendLoadDictStatus
        # отправлять в КЦ подробные статусы о прогрузке справочников
        self.isSendDebugDictStatus = isSendDebugDictStatus
        # адрес сервера, обычно локальный, на который отправляются статусы прогрузки
        self.defaultStatusServer = defaultStatusServer

    def sendDictState(self, messageEn, messageRu, status, tableName='all'):
        self.log.info(u'Отправка статуса загрузки справочника')
        if self.dictCSLoadStatusId and self.isSendLoadDictStatus:
            urlPrefix = 'http://%s%s/dicts' % (self.defaultStatusServer, cs_prefix)
            bodyBase = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status}
            self.log.debug('NES mode: CS')
            tables = tableName if isinstance(tableName, (set, list)) else [tableName]
            for table in tables:
                self.log.info(u'Отправка статуса справочника для таблицы(%s)' % table)
                url, body, method = self._dictLoadCSState(urlPrefix, bodyBase, table)
                self._sendRequest(url, body, method)

        if self.dictCashLoadStatusId and (status != STATUS_DURING or self.isSendDebugDictStatus):
            bodyBase = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status}
            urlPrefix = 'http://%s%s/dicts' % (self.statusServer, cs_prefix)
            self.log.debug('NES mode: cash')
            url, body, method = self._dictLoadCashState(urlPrefix, bodyBase)
            self._sendRequest(url, body, method)

    def _dictLoadCashState(self, urlPrefix, bodyBase):
        dictCashLoadStatusId = self.dictCashLoadStatusId
        if self.version and self.version >= '3':
            dictCashLoadStatusId += '?useDictGenerateId=true'
        url = '%s/cashLoad/%s/%s/%s' % (urlPrefix, self.shopCode, self.cashCode, dictCashLoadStatusId)
        return url, bodyBase, 'PUT'

    def _dictLoadCSState(self, urlPrefix, bodyBase, tableName):
        url = '%s/CSLoad/%s' % (urlPrefix, self.shopCode)
        if tableName:
            url = url + '?tableName=%s' % tableName
        bodyBase['dictBackOffice'] = {'id': self.dictCSLoadStatusId}
        return url, bodyBase, 'POST'

    def _sendRequest(self, url, body, method):
        body = json.dumps(body)
        self.log.info(u"url: '%s', method: '%s', headers: '%s', body: '%s'" % (url, method, jsonHeader, body))
        try:
            http = httplib2.Http(timeout=requestTimeout)
            response, content = http.request(url, method, headers=jsonHeader, body=body)
            if response.status == 200:
                return json.loads(content)
            else:
                self.log.warn(u'Ошибка отправки статуса загрузки справочника. %s' % content)
        except Exception as e:
            self.log.warn(u'Ошибка отправки статуса загрузки справочника. %s' % str(e))
        return None


# запуск генерации справочника на КС
def generateDict(shopCode, dictBackOfficeIds, owner, generateList={}, statusServer='localhost:8080'):
    log = logger.getLogger('Shop', shopCode)
    http = httplib2.Http(timeout=requestTimeout)
    if dictBackOfficeIds:
        for dictBackOfficeId in dictBackOfficeIds:
            url = 'http://%s%s/dicts/CSLoad/%s' % (statusServer, cs_prefix, dictBackOfficeId)
            response = False
            content = 'Empty'
            log.info(u"Запрос генерации справочников(адрес '%s', заголовки '%s')" % (url, jsonHeader))
            try:
                response, content = http.request(url, 'GET', headers=jsonHeader)
            except Exception as e:
                log.warn(u'Ошибка %s' % e)
                log.error(traceback.format_exc())

            log.info(u'response=%s' % response)
            if response and response.status == 200:
                dictCSLoads = json.loads(content)
                for dictCSLoad in dictCSLoads:
                    unloadElementName = dictCSLoad.get('unloadElement', {}).get('name', None)
                    if unloadElementName:
                        if unloadElementName in generateList.get('increment', {}):
                            generateList['increment'][unloadElementName].append('')
                        else:
                            if 'increment' not in generateList:
                                generateList['increment'] = {}
                            generateList['increment'][unloadElementName] = ['']
            else:
                log.warn(u'Ошибка получения элементов выгрузки для запуска генерации справочника. %s' % content)
    generateStates = []
    for generateMode in ['increment', 'full']:
        for unloadElementName in generateList.get(generateMode, {}).keys():
            body = json.dumps({'dictId': unloadElementName, 'full': generateMode == 'full'})
            log.info('body = %s' % body)
            cashIds = set(generateList[generateMode][unloadElementName])
            for cashId in cashIds:
                if cashId:
                    url = 'http://%s%s/tree/elements/%s/dicts?owner=%s' % (statusServer, cs_prefix, cashId, owner)
                # если идентификатор кассы не определен, то запускаем генерацию на магазин
                else:
                    url = 'http://%s%s/tree/elements/%s/dicts?owner=%s' % (statusServer, cs_prefix, shopCode, owner)
                log.info(u"url = '%s', headers = '%s'" % (url, jsonHeader))
                try:
                    response, content = http.request(url, 'POST', headers=jsonHeader, body=body)
                    if response.status == 200:
                        generateStates.append(json.loads(content))
                    else:
                        log.warn(u'Ошибка запуска генерации справочника. %s' % content)
                except Exception as e:
                    log.warn(u'Ошибка запуска генерации справочника. %s' % e)
    return generateStates if len(generateStates) > 0 else None


def getQueueNames(host, brokerRestPort, user, password):
    header = jsonHeader.copy()
    header["Authorization"] = "Basic {0}".format(base64.b64encode("{0}:{1}".format(user, password)))
    url = 'http://%s:%s/rest/queue' % (host, brokerRestPort)
    try:
        http = httplib2.Http(timeout=requestTimeout)
        response, content = http.request(url, 'GET', headers=header)
        if response.status == 200:
            return [str(queue.get('name', '')) for queue in json.loads(content)]
    except Exception:
        log = logger.getLogger('Service')
        log.error(u'Can not get queue names by REST. url = %s' % url)
    return []
