# -*- coding: utf-8 -*-

import mysql.simplesql as sql

tables = ['prices']


def insertPrice(price):
    yield sql.Insert(table='prices', values={
        'barcode': price.get('barcode'),
        'documentid': price.get('documentid', None),
        'effectivedate': getEffectiveDate(price),
        'doctype': price.get('doctype', None),
        'price': price.get('price', None),
        'minprice': price.get('minprice', None),
        'pricetype': price.get('pricetype', None),
        'effectivedateend': price.get('effectivedateend', None),
        'minretailprice': price.get('minretailprice', None),
        'packingprice': price.get('packingprice', None)
    })


def deletePriceByBarcode(bcode, documentid=None, doctype=None):
    yield sql.Delete(table='prices',
            whereFields=['barcode', 'documentid', 'doctype'],
            whereValues=[bcode, documentid, doctype])


def deletePrice(price):
    for i in deletePriceByBarcode(price.get('barcode', None), price.get('documentid', None), price.get('doctype', None)):
        yield i


def clearPrice():
    yield sql.DeleteAll(table='prices')


def deletePriceByDate(bcode, effectivedate=None, effectivedateend=None):
    yield sql.DeleteByDate(table='prices',
            whereFields=['barcode', 'effectivedate', 'effectivedateend'],
            whereValues=[bcode, effectivedate, effectivedateend])


def deleteOverduePrices(bcode):
    yield sql.DeleteOverduePrices(table='prices', whereFields=['barcode'], whereValues=[bcode])


def getEffectiveDate(price):
    # если не заполнено поле effectivedate, по просьбе клиента устанавливаем по-умолчанию 1970-01-02 00:00:00(2 янв, чтобы избежать проблем с часовым поясом)
    # http://redmine.artix.su/issues/48289
    effectivedate = price.get('effectivedate', None)
    if effectivedate is None or '0000-00-00' in effectivedate:
        effectivedate = "1970-01-02 00:00:00"
    return effectivedate
